// Generated with Packemon: https://packemon.dev
// Platform: browser, Support: stable, Format: esm
import { provide, inject } from 'vue';
/**
 * Define a context that can be provided and used in the component hierarchy. Pass in a create
 * function that will be run as part of setup(). All Vue reactive methods such as `computed`
 * may be used.
 *
 * @example
 * export const userName = defineContext(() => {
 *   const userName = ref('');
 *   const uppercaseUserName = computed(() => userName.value.toUpperCase())
 *   const setUserName = (name) => {
 *     userName.value = name;
 *   }
 *   return {
 *     userName: readonly(userName),
 *     uppercaseUserName,
 *     setUserName,
 *   }
 * });
 *
 * @param {NewContextFn<C>} newContextFn Function that creates the context. Make it return an
 * object just as any other setup() function.
 * @returns {ContextDefinition<C>} A context definition object that is used by `provideContext`
 * and `useContext`.
 */

function defineContext(newContextFn) {
  return {
    injectionKey: Symbol(),
    newContextFn: newContextFn
  };
}
/**
 * Create a context that can be provided to all components in the application setup.
 *
 * @example
 * import { userName } from '@contexts/user';
 *
 * createApp(App)
 *   .provide(...createContext(userName))
 *   .mount('#app');
 *
 * @param {ContextDefinition<C>} contextDefinition The context definition object created by
 * `defineContext`.
 * @returns {[InjectionKey<C>, C]} The injection key and the newly created context.
 */


function createContext(_ref) {
  var injectionKey = _ref.injectionKey,
      newContextFn = _ref.newContextFn;
  return [injectionKey, newContextFn()];
}
/**
 * Provide context from a component in the hierarchy. All descendant components can then use
 * that context.
 *
 * @example
 * import { userName } from '@contexts/user';
 *
 * export default defineComponent({
 *   setup() {
 *     provideContext(userName);
 *   },
 * });
 *
 * @param {ContextDefinition<C>} contextDefinition The context definition object created by
 * `defineContext`.
 * @returns {void}
 */


function provideContext(contextDefinition) {
  provide.apply(void 0, createContext(contextDefinition));
}
/**
 * Use context from an ancestor component in the hierarchy.
 *
 * @example
 * import { userName } from '@contexts/user';
 *
 * export default defineComponent({
 *   setup() {
 *     const { uppercaseUserName, setUserName } = useContext(userName);
 *     return {
 *       uppercaseUserName,
 *       setUserName,
 *     }
 *   },
 * });
 *
 * @param {ContextDefinition<C>} contextDefinition The context definition object created by
 * `defineContext`.
 * @returns {C} The context from the first ancestor of this component that provides it.
 */


function useContext(_ref2) {
  var injectionKey = _ref2.injectionKey;
  var context = inject(injectionKey);

  if (!context) {
    throw Error("Could not inject context, was it provided in an ancestor component or from the application?");
  }

  return context;
}

export { createContext, defineContext, provideContext, useContext };
//# sourceMappingURL=index.js.map
